/**
 	@author Rodrigo Shimabuco
		Jaime Henrique F. Queiroz

 */

#include "microblaze_isa.H"
#include "microblaze_isa_init.cpp"
#include "microblaze_bhv_macros.H"

using namespace microblaze_parms;

//! O comportamento (behavior) é executado antes do inicio da simulação.
void ac_behavior(begin) {
	cout << "----- Inicio do comportamento ----- \n\n";
	return;
}

//! O comportamento (behavior) é executado após o fim da simulação.
void ac_behavior(end) {
	cout << "\n\n----- Fim do comportamento ----- \n\n";
	return;
}

//! O comportamento generico de todas as instruções
void ac_behavior(instruction){

	switch( stage ) {
		case id_pipe_IF:
			cout << "\n IF----->ID \n";
			ac_pc = ac_pc + 4;
			IF_ID.npc = ac_pc;
			break;
	
		case id_pipe_ID:
			cout << "\n ID------>EX \n";
			break;
	
		case id_pipe_EX:
			cout << "\n EX------>MEM \n";
			break;

		case id_pipe_MEM:
			cout << "\n MEM----->WB \n";
			break;
	
		case id_pipe_WB:
			cout << "\n ---- WB ----\n";

			// Executa o Writeback quando é permitido.
			if (MEM_WB.regwrite == 1) {
				// Nunca é escrito no registrador 0.
				if (MEM_WB.rdest != 0)
					cout << "\n $R" << MEM_WB.rdest << " = " << MEM_WB.wdata << "\n\n";
					RB.write(MEM_WB.rdest.read(), MEM_WB.wdata.read()); // Escreve no banco de registradores.
			}
			break;

		default:
			break;
	}
};

//! Implementação do comportamento das instruções do Tipo A
void ac_behavior (Tipo_A) {

	switch ( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.regwrite = 1;
			ID_EX.memread = 0;
			ID_EX.memwrite = 0;
			ID_EX.npc = IF_ID.npc;
			ID_EX.ra = ra;
			ID_EX.rb = rb;
			ID_EX.rd = rd;

			// Checando o fowarding para o registrador ra
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.ra) ) {
       				operando1 = EX_MEM.alures.read();
			}
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.ra) &&(EX_MEM.rdest == ID_EX.ra) ) {
					operando1 = MEM_WB.wdata.read();
			}
			else
  				operando1 = RB.read(ra);
			// Checando o fowarding para o registrador rb
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.rb) ) {
       				operando2 = EX_MEM.alures.read();
			}
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.rb) &&(EX_MEM.rdest == ID_EX.rb) ) {
		 		operando2 = MEM_WB.wdata.read();
			}
			else
				operando2 = RB.read(rb);

			ID_EX.data1 = operando1;
			ID_EX.data2 = operando2;
			break;


		case id_pipe_EX:
			// Checando o fowarding para o registrador ra.
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.ra) )
				valor1 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.ra) )
				valor1 = MEM_WB.wdata;
			else
				valor1 = ID_EX.data1;

			// Checando o fowarding para o registrador rb.
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.ra) )
				valor2 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.ra) )
				valor2 = MEM_WB.wdata;
			else
				valor2 = ID_EX.data2;
			break;

		case id_pipe_MEM:
			MEM_WB.regwrite = EX_MEM.regwrite;
			MEM_WB.wdata = EX_MEM.alures;
			MEM_WB.rdest = EX_MEM.rdest;
			break;

		case id_pipe_WB:
			break;
		default:
			break;
	}

};


// Implementação do comportamento das instruções do Tipo B.
void ac_behavior (Tipo_B) {

	switch ( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.regwrite = 1;
			ID_EX.memread = 0;
			ID_EX.memwrite = 0;
			ID_EX.npc = IF_ID.npc;
			ID_EX.ra = ra;
			ID_EX.rd = rd;
			ID_EX.imm = imm;

			// Checando o fowarding para o registrador ra
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.ra) )
				operando1 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.ra) )
				operando1 = MEM_WB.wdata;
			else 
				operando1 = ID_EX.ra;

			/// Checando o fowarding para o registrador rd (instruções de desvio)
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.rd) )
				operando2 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.rd) )
				operando2 = MEM_WB.wdata;
			else
				operando2 = ID_EX.rd;
    
			ID_EX.data1 = operando1;
			ID_EX.data2 = operando2;

			break;

		case id_pipe_EX:
			// Checando o fowarding para o registrador rd
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.rd) )
				valor1 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.rd) )
				valor1 = MEM_WB.wdata;
			else
				valor1 = ID_EX.data1;

			// Checando o fowarding para o registrador ra (instruções de desvio)
			if ( (EX_MEM.regwrite == 1) && (EX_MEM.rdest != 0) && (EX_MEM.rdest == ID_EX.ra) )
				valor2 = EX_MEM.alures;
			else if ( (MEM_WB.regwrite == 1) && (MEM_WB.rdest != 0) && (MEM_WB.rdest == ID_EX.ra) )
				valor2 = MEM_WB.wdata;
			else
				valor2 = ID_EX.data2;

		case id_pipe_MEM:
			MEM_WB.regwrite = EX_MEM.regwrite;
			MEM_WB.wdata = EX_MEM.alures;
			MEM_WB.rdest = EX_MEM.rdest;
			break;

		case id_pipe_WB:
			break;
		default:
			break;
	}

};

void ac_behavior( add )
{
	cout << "  add\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = valor1 + valor2; // Soma valor de ra+rb;

			// Teste para ver se deu overflow na soma
			if ( ((valor1 & 0x80000000) == (EX_MEM.alures & 0x80000000)) && ((EX_MEM.alures & 0x80000000) != (valor2 & 0x80000000)) ) {
				fprintf(stderr, "EXCEPTION(add): integer overflow.\n");
				exit(EXIT_FAILURE);
			}
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( addi )
{
	cout << "  addi\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = valor1 + ID_EX.imm; // Soma o valor do ra + imeditado.

			// Checando overflow
			if ( ((valor1 & 0x80000000) == (ID_EX.imm & 0x80000000)) && ((ID_EX.imm & 0x80000000) != (valor2 & 0x80000000)) ) {
				fprintf(stderr, "EXCEPTION(addi): integer overflow.\n");
				exit(EXIT_FAILURE);
			}

			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;
		case id_pipe_MEM:
			break;
		case id_pipe_WB:
			break;
		default:
			break;
	}
};

void ac_behavior( and )
{
	cout << "  and\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra & ID_EX.rb; // Faz o and entre ra e rb.
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( andi )
{
	cout << "  andi\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra & (ID_EX.imm & 0xFFFF); // Faz o and entre ra e imm extendido para 32 bits.
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( andn )
{
	cout << "  andn\n";
	switch( stage ) {
		case id_pipe_IF:
			break;
		case id_pipe_ID:
			break;
		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra & !(ID_EX.rb); // Faz o and entre ra e rb(negado).
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;
		case id_pipe_MEM:
			break;
		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( andni )
{
	cout << "  andni\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra & !(ID_EX.imm & 0xFFFF); // Faz o and entre ra e imm extendido para 32 bits (negado).
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( beq )
{
	cout << "  beq\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for beq (rd == 0) e ra == 0 então faça pc += rb
			if (ID_EX.rd == 0 && ID_EX.ra == 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb; 
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( beqi )
{
	cout << "  beqi\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for beqi (rd===0) e ra == 0, então faça pc += imm;
			if (ID_EX.rd == 0 && ID_EX.ra == 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}			
};

void ac_behavior( bge )
{
	cout << "  bge\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bge (rd == 5) e ra >= 0, então faça pc += rb;
			if (ID_EX.rd == 5 && ID_EX.ra >= 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( bgei )
{
	cout << "  bgei\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bgei (rd == 5) e ra >= 0, então faça pc += imm;
			if (ID_EX.rd == 5 && ID_EX.ra >= 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( bgt )
{
	cout << "  bgt\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bgt (rd == 4) e ra > 0, então faça pc += rb;
			if (ID_EX.rd == 4 && ID_EX.ra > 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( bgti )
{
	cout << "  bgti\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bgti (rd == 4) e ra > 0, então faça pc += imm;
			if (ID_EX.rd == 4 && ID_EX.ra > 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( ble )
{
	cout << "  ble\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for ble (rd == 3) e ra > 0, então faça pc += rb;
			if (ID_EX.rd == 3 && ID_EX.ra <= 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( blei )
{
	cout << "  blei\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for blei (rd == 3) e ra > 0, então faça pc += imm;
			if (ID_EX.rd == 3 && ID_EX.ra <= 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( blt )
{
	cout << "  blt\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for blt (rd == 2) e ra > 0, então faça pc += rb;
			if (ID_EX.rd == 2 && ID_EX.ra < 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( blti )
{
	cout << "  blti\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for blti (rd == 2) e ra > 0, então faça pc += imm;
			if (ID_EX.rd == 2 && ID_EX.ra < 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( bne )
{
	cout << "  bne\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bne (rd == 1) e ra > 0, então faça pc += rb;
			if (rd == 1 && ID_EX.ra != 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.rb;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( bnei )
{
	cout << "  bnei\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			// Se for bnei (rd == 1) e ra > 0, então faça pc += imm;
			if (ID_EX.rd == 1 && ID_EX.ra != 0) {
				ID_EX.npc = ID_EX.npc + ID_EX.imm;
			} else { // Se não, pc += 4;
				ID_EX.npc = ID_EX.npc + 4;
			}
			break;

		case id_pipe_EX:
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( lw )
{
	cout << "  lw\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.memread = 1; // Seta o memread para 1, ou seja, vai ler da memória.
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra + ID_EX.rb; // Carrega o valor de ra+rb em alures (resultados da ALU).
			EX_MEM.regwrite = ID_EX.regwrite;    
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			MEM_WB.wdata = DM.read(EX_MEM.alures);
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( lwi )
{
	cout << "  lwi\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.memread = 1; // Seta memread como 1, ou seja, vai ler da memória.
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra + ID_EX.imm; // Soma ra + imediato e coloca em alures.
			EX_MEM.regwrite = ID_EX.regwrite;    
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			MEM_WB.wdata = DM.read(EX_MEM.alures);
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( or )
{
	cout << "  or\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra | ID_EX.rb; // Faz o or entre ra e rb;
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( ori )
{
	cout << "  ori\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			break;

		case id_pipe_EX:
			EX_MEM.alures = ID_EX.ra | ID_EX.imm; // faz o or entre ra e imediato.
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( nop )
{
	cout << "  NOP\n";
	return;
};

void ac_behavior( sw )
{
	cout << "  sw\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.memwrite = 1; // seta 1 para escrita na memória
			ID_EX.regwrite = 0; // seta 0 para não escrita no banco de registradores.
			break;

		case id_pipe_EX:
			EX_MEM.alures = valor1 + valor2; // armazena o valor de ra+rb em alures.
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			DM.write(EX_MEM.alures, EX_MEM.wdata); // escreve na memória o valor;
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

void ac_behavior( swi )
{
	cout << "  swi\n";
	switch( stage ) {
		case id_pipe_IF:
			break;

		case id_pipe_ID:
			ID_EX.memwrite = 1; // seta 1 para escrita na memória.
			ID_EX.regwrite = 0; // seta 0 para escrita no banco de registradores.
			break;

		case id_pipe_EX:
			EX_MEM.alures = valor1 + ID_EX.imm; // Soma o valor de ra + imediato e coloca em alures.
			EX_MEM.regwrite = ID_EX.regwrite;
			EX_MEM.memread = ID_EX.memread;
			EX_MEM.memwrite = ID_EX.memwrite;
			EX_MEM.rdest = ID_EX.rd;
			break;

		case id_pipe_MEM:
			DM.write(EX_MEM.alures, EX_MEM.wdata); // Escreve na memória.
			break;

		case id_pipe_WB:
			break;

		default:
			break;
	}
};

